{
This is a component for placing icons in the notification area
of the taskbar.

The source is freeware. (you may even remove the about property)

See the included example on how to use the component.

Any corrections/hints are welcome on j.h.dewaalmalefijt@wbmt.tudelft.nl

Have fun,
JH
}

unit Traybar;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  dsgnintf,
  Dialogs,
  ShellApi,
  Menus;

const
   {Userdefined message sent from the icon. Some low user defined
    messages are used by windows itself! (WM_USER+1=DM_SETDEFID)}
   WM_MYMESSAGE = WM_USER + 1024;
   IconID       = 1;

type
  {All this misery for only an aboutbox:}
  TAboutTraybarProperty = class(TPropertyEditor)
  public
     procedure Edit; override;
     function GetAttributes: TPropertyAttributes; override;
     function GetValue: string; override;
  end;
  {The real component:}
  TTraybar = class(TComponent)
  private
    {Hiding implementation details,}
    {Properties storage}
    FAbout: TAboutTraybarProperty;
    FEnabled: Boolean;
    FICon: TIcon;
    FHint: String;
    FName: TComponentName;
    Tnid: TNotifyIconData;
    FPopupMenu: TPopupMenu;
    FOnClick,
    FOnDblClick,
    FOnRightClick: TNotifyEvent;
    function InitIcon: Boolean;
    procedure SetIcon(Value: TIcon);
    procedure HandleMessage(var Msg: TMessage);
  protected
    {Defining the developers interface}
  public
    {Defining the run-time interface}
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
    function Add: Boolean;
    function Modify: Boolean;
    function Delete: Boolean;
  published
    {Defining the design time interface}
    {New properties:}
    property About: TAboutTraybarProperty read FAbout write FAbout;
    property Hint: String read FHint write FHint;
    property Icon: TIcon read FIcon write SetIcon stored True;
    property PopupMenu: TPopupMenu read FPopupMenu write FPopupMenu;
    {New events:}
    property Onclick: TNotifyEvent read FOnClick write FOnClick;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnRightClick: TNotifyEvent read FOnRightClick write FOnRightClick;
  end;

procedure Register;

implementation

{
About box
}
procedure TAboutTraybarProperty.Edit;
begin
   ShowMessage('This component is freeware'+Chr(13)+'WorldWare 1997');
end;

function TAboutTraybarProperty.GetAttributes: TPropertyAttributes;
begin
   Result := [paDialog, paReadOnly];
end;

function TAboutTraybarProperty.GetValue: string;
begin
   Result := '(about, press for info)';
end;

{
When creating declare never changing fields of tnid
}
constructor TTraybar.Create( AOwner: TComponent );
begin
   inherited Create( AOwner );
   FIcon := TIcon.Create;
   tnid.cbSize := SizeOf(TNotifyIconData);
   {tnid.wnd points to procedure to receive callback messages
   form the icon}
   tnid.wnd := AllocateHWnd(HandleMessage);
   {Just a number for the new icon}
   tnid.uId := IconID;
   {Icon, message en tiptext members are/myst be valid}
   tnid.uFlags := NIF_ICON + NIF_MESSAGE + NIF_TIP;
   {message to send to tnid.wnd when mouse event occurs}
   tnid.uCallbackMessage := WM_MYMESSAGE;
end;

{
Free FIcon and go on with Destroy from TComponent
}
destructor TTraybar.Destroy;
begin
   FIcon.Free;
   inherited Destroy;
end;

{
Assign icon to FIcon
}
procedure TTraybar.SetIcon(value: TIcon);
begin
   FIcon.Assign(value);
end;

{
Handle incoming messages from the traybaricon.
All known traybaricons (like speaker, FlexiCD, Clock etc)
react on mousebutton actions as used below
}
procedure TTraybar.HandleMessage(var Msg: TMessage);
var
   CursorPos: TPoint;
begin
   With msg do
   {Only take action if a message from the icon comes through}
   If Msg=WM_MYMESSAGE then
      Case LParam of
         {Call event only when a procedure is assigned to it!}
         WM_LBUTTONDBLCLK: If Assigned(FOnDblClick) then
                              FOnDblClick(Self);
         WM_LBUTTONDOWN: If Assigned(FOnClick) then
                            FOnClick(Self);
         {If an event assigned to OnRightClick the popupmenu
          never shows!}
         WM_RBUTTONUP: If Assigned(FOnRightClick) then
                          FOnRightClick(Self)
                       else
                          If Assigned(PopupMenu) then
                             If GetCursorPos(CursorPos) then
                                PopupMenu.Popup(CursorPos.X, CursorPos.Y);

      end;
end;

{
Fill tnid with changing date, like icon and tooltip
}
function TTraybar.InitIcon: Boolean;
begin
   If Not(CsDesigning in ComponentState){Not in Designmode} and
      Not(CsLoading in ComponentState) {And not loading component} then
      {Above checks aren't really necssary}
      begin
         tnid.hIcon := Icon.Handle;
         If Hint<>'' then
            StrlCopy(tnid.szTip, PChar(Hint), sizeof(tnid.szTip))
         else
            tnid.szTip := '\0';
         Result := True;
      end
   else
      Result := False;
end;

{
Adds an icon, Result=True if successful
}
function TTraybar.Add: Boolean;
begin
   If InitIcon then
      Result := Shell_NotifyIcon(NIM_ADD, @tnid);
end;

{
Change icon or tooltip if icon already placed
}
function TTraybar.Modify: Boolean;
begin
   If InitIcon then
      Result := Shell_NotifyIcon(NIM_MODIFY, @tnid);
end;

{
Remove icon
}
function TTraybar.Delete: Boolean;
begin
   If InitIcon then
      Result := Shell_NotifyIcon(NIM_DELETE, @tnid);
end;


procedure Register;
begin
  RegisterComponents('WorldWare', [TTraybar]);
  RegisterPropertyEditor(TypeInfo(TAboutTraybarProperty), TTraybar, 'ABOUT', TAboutTraybarProperty);
end;

end.

{That's about it!}
